/********************************************************************************************
 *RFXtrxEmu v0.15
 *
 *Die Demo-Firmware RFXtrxEmu emuliert das AC-Protokoll des RFXtrx433 Funk-Gateways der Fa. RFXCOM.
 *Auf AKtron-IO geflasht kann es als kabelgebundene Loesung an ein Hausautomatisierungs-
 *system wie z.B. Domoticz angebunden werden.
 *Weil AKtron-IO mehr Ein-/Ausgänge als das Arduino Mega2560 Board hat, wird auf
 *digitalRead() und digitalWrite() verzichtet und stattdessen Port-/Byteweise gelesen
 *und geschrieben.
 *
 *IDE:  Arduino 1.8.x
 *Mikrocontroller:  ATmega2560
 *Boardauswahl : Arduino AVR Boards -> Mega 2560
 ********************************************************************************************/
/********************************************************************************************
 * Die hier unter GNU General Public Lizenz v3 kostenlose und zur freien Verfuegung gestellte
 * OSS (Open Source Software) hat den Zweck dem Nutzer den Einstieg zu erleichtern und kann 
 * als Grundlage für eigene Erweiterungen, Projekte oder Schulungszwecke verwendet werden.
 * Bei den Softwareversionen 0.x handelt es sich um Demo-Versionen bei denen für Demozwecke
 * von „ausreichendem“ Funktionsumfang auszugehen ist.
 * Es besteht KEIN rechtlicher Anspruch auf Vollständigkeit und Fehlerfreiheit.
 *
 * Copyleft Andreas Kaestner https://aktron.de
 ********************************************************************************************/



// BOF preprocessor bug prevent - insert me on top of your arduino-code
#if 1
__asm volatile ("nop");
#endif


// Definitionen für Portfunktion
#define EinAus  0  // Eingangsport der einen Ausgangsport schaltet
#define Ausg    1  // Ausgangsport
#define Eing    2  // Eingang (z.B Fensterkontakt) ohne Schaltzuweisung zu einem Ausgangsport

// Definitionen für Modus
#define Frei      0 // Wird nicht von einem Eingangsport geschaltet 
#define Schalter  1
#define Motor     2

#define  EE_DeviceID    152
#define  EE_Twi         153 // Twi (I2C) [0...1]

#define  EE_Inverse     155 // ... 162
#define  EE_InverseSend 163 // ... 170
#define  EE_FlagTime    256 // ... 336
#define  EE_Password    337 // ... 352
#define  EE_SMS1        353 // ... 368
#define  EE_SMS2        369 // ... 384

#define  EE_UseAlarm    385
#define  EE_UsePLC      386

#define  EE_PLCend      3070     
#define  EE_PLC         3072

#define  PreAlarmDura     100 // 1/10 Sekunden Voralarmdauer 

#include <avr/wdt.h>
#include <EEPROM.h>
#include "Adafruit_GFX.h"
#include "SH1106.h"


#define OLED_RESET 4
SH1106 display(OLED_RESET);

#define XPOS 0
#define YPOS 1

#define _DisplayXPosID 115

#if (SH1106_LCDHEIGHT != 64)
 #error("Height incorrect, please fix SH1106.h!");
#endif

// On board BlueLED
#define LED_ON  PORTJ|=128;
#define LED_OFF PORTJ&=127;

String  sVersion = " 0.15";

String sPassword;
String SMS_Number1;
String SMS_Number2;

String SMS_Msg    = "Achtung! Alarm aktiviert!"; 

byte    bBoardType = 0;
 
unsigned int iSMS_SeqTimer  = 0;   // Timer für Zeitabstände zwischen den AT-SMS Kommandos
unsigned int iSMS_Timer     = 0;   // Zeit zwischen 1. und 2. SMS
byte         bSMS_Cnt       = 2;   // Anzahl der SMSn die gesendet werden sollen
byte         bSMS_Act       = 0;   // Aktuell zu sendene SMS   


byte bDeviceID, bP, bX;            // bDeviveID 0...F, bP-Port 0...7, bX-Bit 0...7
boolean xOnOff;
boolean xTest = false;
String sCmd, sPlain;

boolean xUseAlarm = false;  
boolean xUsePLC   = false;    

boolean xTwi   = 1;

// SPS Variablen
boolean xAllocation = false;
boolean xState = false;          // Alarmanlage 0 = unscharf| 1 = scharf
byte bAlarm = 0;                 // Alamierung  0 = aus | 1 = Voralarm | 2 = Hauptalarm
unsigned int tPreAlarm = 0;  
unsigned int tMainAlarm = 0;
unsigned int tLED_on = 300;      // So wird beim Starten der Firmware die LED am Reader nach 30s ausgeschaltet.
boolean xInter1010 = false;
boolean xInter5050 = false;
boolean xExtentedPreAlert = false;
boolean xLED_on = false;
char cSlaveID = '1';

byte br1, br2;
byte bFlagNbr; // Merkeradresse





byte cPLC[512];
unsigned int iPLCend = 0;


struct PortFnc
{
 byte Fnc; // 0 = Eingangsport der Ausgangsport schaltet, 1 = Ausgangsport, 2 = kontaktabfragender Eingangsport
 byte Ass;
};

PortFnc Port[8];

// Emulator-Kommandos für Hausautomatisierungs- und Web-Visualisierungs- Systeme ( Domoticz, Home Assistant, openHab, Fhem usw.)
// Hier wird das Kommunikationsprotokoll vom dem RFXtrx 433 Gateway emuliert.
// Das RFXtrx 433 Gateway wird durch AKtron-IO ersetzt. Statt Funk-Schalter und Funk-Steckdosen werden die Eingänge und
// Ausgänge (Relais) verwendet.
// Durch die Emulation ist bei den Hausautomatisierungs-Systemen KEIN Update oder Modul-Implementierung erforderlich
// weil das RFXtrx 433 Gateway als Standard bekannt ist ;-)

byte    RxVisuIni[16]   = {0x0d, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
byte    TxVisuIni[16]   = {0x0d, 0x01, 0x00, 0x01, 0x02, 0x53, 0x46, 0x00, 0x00, 0x06, 0x01, 0x01, 0x00, 0x00};
byte    TxVisu[16]      = {0x0b, 0x11, 0x00, 0x00, 0x00, 0xAA, 0xBB, 0x00, 0x00, 0x01, 0x0F, 0x70};

//Antwort auf Abfrage   0D 00 00 [01] 07 00 00 00 00 00 00 00 00 00  ->[Zähler]
byte    Tx07[21]      = {0x14, 0x01, 0x07, 0x01, 0x07, 0x43, 0x6F, 0x70, 0x79, 0x72, 0x69, 0x67, 0x68, 0x74, 0x20, 0x52, 0x46, 0x58, 0x43, 0x4F, 0x4D};

//Antwort auf Abfrage   0D 00 00 [02] 02 00 00 00 00 00 00 00 00 00  ->[Zähler]
byte    Tx02[14]      = {0x0D, 0x01, 0x00, 0x02, 0x02, 0x53, 0x5D, 0x00, 0x00, 0x04, 0x00, 0x01, 0x01, 0x1C};


boolean xQ   [8][8];    // aktueller Zustand Ausgang
// Wird für SPS benötigt
boolean xQPre[8][8];    // vorheriger Zustand Ausgang
boolean xQR  [8][8];    // Flanke positiv (rise)
boolean xQF  [8][8];    // Flanke negativ (fall)
boolean xQC  [8][8];    // PinChange
// Merker
boolean xM   [128];    // aktueller Zustand Merker
// Wird für SPS benötigt 
boolean xMPre[128];    // vorheriger Zustand Merker
boolean xMR  [128];    // Flanke positiv (rise)
boolean xMF  [128];    // Flanke negativ (fall)
boolean xMC  [128];    // PinChange
//

byte    bQMode  [8][8];     // 0 = Kontakt(kein Ausgang), 1 = Schalter, 2 = Motor

//Ports 0-7
byte    bI[8]             = {0,0,0,0,0,0,0,0};  // aktueller Zustand Eingang
byte    bIPre[8]          = {0,0,0,0,0,0,0,0};  // Vorheriger Zustand Eingang
byte    bIR[8]            = {0,0,0,0,0,0,0,0};  // Flanke positiv (rise)
byte    bIF[8]            = {0,0,0,0,0,0,0,0};  // Flanke negativ (fall)
byte    bIC[8]            = {0,0,0,0,0,0,0,0};  // PinChange

byte    bReverse[8]       = {0,0,0,0,0,0,0,0};  // Port-Pins revers behandeln ( 0 = normal, 1 = umgekehrt bzw. gespiegelt)
byte    bInverse[8]       = {0,0,0,0,0,0,0,0};  // Port invers behandeln ( 0 = normal, 1 = invertiert)
byte    bInverseSend[8]   = {0,0,0,0,0,0,0,0};  // Status ON/OFF invertiert senden (Bei Domoticz ist OFF = Fenster geschlossen) ( 0 = normal, 1 = invertiert)


unsigned long  tQOn[8][8];                  // Ablaufende Einschaltzeit
unsigned long  tQOnDelay[8][8];             // Umschaltverzögerung Motor
byte           bQOnSet[8][8];               // Einschaltzeit/dauer 

unsigned long  tMOnRun[128];                // Ablaufende Einschaltzeit
byte           bMOnSet[128];                // Einschaltzeit/dauer 


unsigned long pre100ms = 0;

//Rx0
char          Rx0[1024];
unsigned int  Len0 = 0; // int weil SPS-String 1024 Zeichen lang sein kann
boolean       Rx0TimeOver = true;
unsigned long cTime0; // aktuelle Zeit
unsigned long pTime0 = 0; // vorherige Zeit

//Rx1 
char          Rx1[64];
byte          Len1 = 0;
boolean       Rx1TimeOver = true;
unsigned long cTime1; // aktuelle Zeit
unsigned long pTime1 = 0; // vorherige Zeit


//Rx2
char          Rx2[32];
byte          Len2 = 0;
boolean       Rx2TimeOver = true;
unsigned long cTime2; // aktuelle Zeit
unsigned long pTime2 = 0; // vorherige Zeit

long          interval = 10;           //  (milliseconds)


void setup() {

byte i;  

 DDRG  &=  B11111011; // Eingang PORTG.2 Board-Check 32/64
 PORTG |=  B00000100;  // Pullup setzen
 
 DDRJ  |=  B10000000; // Status LED Ausgang PORTJ.7
 

 LED_ON  //PIN J7 einschalten
 delay(200);
 LED_OFF  //PIN J7 ausschalten
 delay(200);
 LED_ON;  //PIN J7 einschalten
 delay(200);
 LED_OFF  //PIN J7 ausschalten 
 
  // Welche Hardware-Variante (BoardType)?
  if((PING & B00000100) > 0)
   bBoardType = 32;
  else
   bBoardType = 64; 

  // Serielle Hardware-Schnittstellen initialisieren:
  Serial.begin (38400);        //  USB-Port zu uebergeordnetem Device HA-Server Terminal
  Serial1.begin(38400);        //  RS485 nur HA-Server ohne Terminal 
  Serial2.begin(9600);         //  Transponder-Terminals
  Serial3.begin(9600);         //  GSM-Modul um SMS zu senden TTL
      
 display._i2caddr = 0x3C;     //   je nach Displayhersteller auch 0x3D

//************* SPS Programm Test *********************
//PLC 

//Hi nibble
#define _l     0         //  0 lade
#define _ln    1 * 16    // 16 lade nicht    
#define _u     2 * 16    // 32 und
#define _un    3 * 16    // 48 und nicht
#define _o     4 * 16    // 64 oder
#define _on    5 * 16    // 80 oder nicht
#define _x     6 * 16    // 96 xor
// Zuweisungen
#define _ist   8 * 16    // 128 absolute Zuweisung des VKE
#define _istn  9 * 16    // 144 absolute Zuweisung des invertierten VKE
#define _e     10 * 16   // 160 wenn VKE 1 dann Zuweisung Operand EIN
#define _a     11 * 16   // 176 wenn VKE 1 dann Zuweisung Operand AUS
#define _i     12 * 16   // 192 wenn VKE 1 dann Zuweisung Operand invertieren

//Low nibble
// keine Flanke = 0
#define _s    1          // steigende Flanke
#define _f    2          // fallende Flanke
#define _w    3          // wechselnde Falnke ( s oder f)


iPLCend =  0 ;

//************* SPS Test-Programm Ende *********************

 for(bP=4; bP <=7; bP++)
  {
  Port[bP].Fnc = Ausg; // Port 4...7 sind immer Ausgänge
  Port[bP].Ass = 0xFF;
    
   for(bX=0; bX <= 7; bX++)
   {
    tQOn[bP][bX]          = 0;
    tQOnDelay[bP][bX]     = 0;
  //bQOnSet[bP][bX]       = 0;
    xQ   [bP][bX]         = 0;
    
    xQPre[bP][bX]         = 0;
    xQR  [bP][bX]         = 0;
    xQF  [bP][bX]         = 0;
    xQC  [bP][bX]         = 0;
 
    bQMode[bP][bX]        = EEPROM.read((bP*8) + bX);       // min. ADR 0
    bQOnSet[bP][bX]       = EEPROM.read((bP*8) + 64 + bX);  // max. ADR 127

    //Wenn noch nicht im EEProm gesetzt wurde (nach erstem flashen der Firmware)

    if( bQMode[bP][bX] < Schalter || bQMode[bP][bX] > Motor )
     bQMode[bP][bX] = Schalter; 
      
    if(bQOnSet[bP][bX] > 0x6F)
      bQOnSet[bP][bX] = 0;
   
   }
  }

  
 for(bP=0; bP <=3; bP++)
  {  
   Port[bP].Fnc = EEPROM.read((bP*2) + 128);    // min. ADR 128
   Port[bP].Ass = EEPROM.read((bP*2) + 128+1);  // max. ADR 143

   // Eingänge
    if(Port[bP].Fnc != EinAus && Port[bP].Fnc != Eing)
    {
     Port[bP].Fnc = Eing;
     Port[bP].Ass = 0x0E;  
    } 

  if(Port[bP].Fnc == Eing)
   Port[bP].Ass = 0x0E;
   
  } 


 for(bP=0; bP <=7; bP++)
  {     
   bReverse[bP] = EEPROM.read(bP + 144);       //min. ADR = 144 | max. ADR 151  
   
   if(bReverse[bP] > 1)
    bReverse[bP] = 0; 

   bInverse[bP] = EEPROM.read(bP + EE_Inverse);       // max. ADR + 7  
   
   if(bInverse[bP] > 1)
    bInverse[bP] = 0;  

   bInverseSend[bP] = EEPROM.read(bP + EE_InverseSend);       // max. ADR + 7  
   
   if(bInverseSend[bP] > 1)
    bInverseSend[bP] = 0;  




       
  }
  
for(i=0; i<=127; i++)
{
  xM       [i]       = 0;
  xMPre    [i]       = 0;
  xMR      [i]       = 0;
  xMF      [i]       = 0;
  xMC      [i]       = 0;

  tMOnRun  [i]       = 0;              // Ablaufende Einschaltzeit
  bMOnSet  [i]       = 0; 
     
} 

for(i=0; i<=80; i++)
{
  bMOnSet  [i] = EEPROM.read(EE_FlagTime + i);
}
   // DeviceID lesen
   bDeviceID = EEPROM.read(EE_DeviceID);
   
   if(bDeviceID > 0x0F)
    bDeviceID = 0x0F;

   // Alarmfunktion lesen
   xUseAlarm = (boolean)EEPROM.read(EE_UseAlarm); 

   if( xUseAlarm > 1)
    xUseAlarm = false;

   // SPS-Funktion lesen
   xUsePLC = (boolean)EEPROM.read(EE_UsePLC); 

   if( xUsePLC > 1)
    xUsePLC = false; 

   xTwi = (boolean)EEPROM.read(EE_Twi);
   
   if(xTwi > 1)  // kann wegen Datentyp [bool] nicht größer 1 sein
    xTwi = true;

 if(xTwi)
 {
  display.TWIInit();
  display.xTwiBegin = true;
  
  while(display.xTwiBegin)
  {
   display.TwiBeginSimultan();
  }
  
  IniGfxRam();

 }    

 //******************************************************
 // EEPROM testweise beschreiben
 /*
  int len;

  sPassword = "Master";
  SMS_Number1 = "0157********";
  SMS_Number2 = "0163*******";

  WriteStringToEE(EE_Password,  sPassword);
  WriteStringToEE(EE_SMS1,  SMS_Number1);  
  WriteStringToEE(EE_SMS2,  SMS_Number2); 
  */ 
  
 


//*****************************************************

  // Passwort lesen
  sPassword =  ReadStringFromEE(EE_Password);
  sPassword.trim();
  //Serial.println(sPassword);

  // 1. SPS Nummer lesen
  SMS_Number1 =  ReadStringFromEE(EE_SMS1);
  SMS_Number1.trim();
  //Serial.println(SMS_Number1);

  // 2. SPS Nummer lesen
  SMS_Number2 =  ReadStringFromEE(EE_SMS2);
  SMS_Number2.trim();
  //Serial.println(SMS_Number2);
  
  // Endadresse vom SPS-Programm lesen
  iPLCend  =  EEPROM.read(EE_PLCend);  // Low Byte
  iPLCend += (EEPROM.read(EE_PLCend+1) << 8);  // High Byte

  if(iPLCend > 511)
   iPLCend = 0;
   
   // SPS-Programm lesen 
   for(i=0; i<=iPLCend; i++)
   {
    cPLC[i] = EEPROM.read(EE_PLC + i);
   }
   
 IniDDR();   
 delay(500);

 WatchDogSetup();
}


/*******************************************
/* String ais EEPROM lesen (max 15 zeichen + NULL)
/*******************************************/
String ReadStringFromEE( unsigned int EE_ADR)
{
 String s = ""; 
 char c;
 int i;
 
   for( i=0; i<=15; i++)
   {
    c = (char)EEPROM.read(EE_ADR + i);
    
    if(c == 0)
     i = 16;  // Schleife verlassen  
    else
     s += c; 
     
   }
 return s;
}

/*******************************************
/* String in EEPROM schreiben (max 15 zeichen + NULL)
/*******************************************/
void WriteStringToEE( unsigned int EE_ADR, String s)
{
 int i;
 char cbuf[16];
 byte len;

  len = s.length();

  if(len > 15)
   len = 15;

  s.toCharArray(cbuf,len+1);

    
   for(i=0; i<=len; i++)
    EEPROM.write(EE_ADR + i, cbuf[i]);
      
    EEPROM.write(EE_ADR + 15, 0); // terminieren 
}


/*******************************************
/* Hauptschleife
/*******************************************/
void loop() { 

 wdt_reset();  
 cTime2 = cTime1 = cTime0 = millis();
 
  //Overflow behandeln
  if( cTime0 < pre100ms)
   pre100ms  =  cTime0;
   
   if(cTime0 - pre100ms >= 50)
   {
    pre100ms  =  cTime0;
    Fnc50ms();
   }

   
 if(!Rx0TimeOver){
    if(cTime0 - pTime0 > interval) {
     pTime0 = cTime0;
     Rx0TimeOver = true;
     CheckRx0();
    }
 }
 else
  pTime0 = cTime0 = millis();
  
  
 if(!Rx1TimeOver){
    if(cTime1 - pTime1 > interval) {
     pTime1 = cTime1;
     Rx1TimeOver = true;
     CheckRx1();
    }
 }
 else
  pTime1 = cTime1 = millis();  


 if(!Rx2TimeOver){
    if(cTime2 - pTime2 > interval) {
     pTime2 = cTime2;
     Rx2TimeOver = true;
     CheckRx2();
    }
 }
 else
  pTime2 = cTime2 = millis();

if(xTwi)
{
 if(display.xTwiBegin)
 {
  display.TwiBeginSimultan();
  display.xIsTwiBeginRunning = true;
 }
 else
 {
  if(display.xIsTwiBeginRunning)
  {
   display.xIsTwiBeginRunning = false;
   IniGfxRam(); 
  }
  
 WriteGfxBuffer();
 display.WriteToDisplay();   
 
 }  
} 

}

/********************************************/
void InputLogic()
{
  for(bP=0; bP<=7; bP++)
  {
   // Feststellen ob der Port ein Eingang und mit Zuweisung auf Ausgang ist
   if(Port[bP].Fnc == EinAus && Port[bP].Ass != 0x0E )
   {    
    //SendInput(); //2014.10.01 Damit zur Weiterverarbeitung auch der Zustand diese Eingänge gesendet wird    
    InToOut();
   }
 
   // Feststellen ob der Port ein Ausgang ist
   if(Port[bP].Fnc == Ausg )
   {      
    ToOut();
   }
   
 // Feststellen ob der Port ein Eingang ist und nur einen Kontakt abfragt also keine Zuweisung auf einen Ausgang hat)  
   if(Port[bP].Fnc == Eing && Port[bP].Ass == 0x0E )
   {
    SendInput();  
   }
  }
}


void serialEvent() {
 // while (Serial.available()) {
    Rx0[Len0++] = Serial.read(); 
    // add it to the inputString:
       // digitalWrite(ledPin, true);
    pTime0 = cTime0 = millis(); // Nachdem ein Byte Empfangen wurde, wird der Timer zurückgesetzt   
    Rx0TimeOver = false;

    // handelt es sich um ein SPS-String beginnend mit "{" ?

    if(Rx0[0] == '{')
    {
     if(Len0 > 1023)
     {
      Len0 = 0;
      Rx0[0] = 0;
     }
    }
    else
    {
    if(Len0 > 63)
     Len0 = 0;
    }
}

void serialEvent1() {
 // while (Serial.available()) {
    Rx1[Len1++] = Serial1.read(); 
    // add it to the inputString:
       // digitalWrite(ledPin, true);
    pTime1 = cTime1 = millis(); // Nachdem ein Byte Empfangen wurde, wird der Timer zurückgesetzt   
    Rx1TimeOver = false;

    if(Len1 > 63)
     Len1 = 0; 
}

void serialEvent2() {
 // while (Serial2.available()) {
    Rx2[Len2++] = Serial2.read(); 
    
 
    pTime2 = cTime2 = millis(); // Nachdem ein Byte Empfangen wurde, wird der Timer zurückgesetzt   
    Rx2TimeOver = false;
    
    if(Len2 > 31)
     Len2 = 0;
}



unsigned long ByteToTime(byte b){
unsigned long t = 0; // Zeit in 100ms
byte scale;

scale = b>>4;

if(scale == 1)// 100 ms
 t = 1;

if(scale == 2)// 1 s
 t = 10;
 
if(scale == 3)// 10 s
 t = 100;

if(scale == 4)// 1 Min
 t = 600;
 
if(scale == 5)// 10 Min
 t = 6000;
 
if(scale == 6)// 1h
 t = 36000; 
 
 
 t *= (b & 0x0F);
 
 return t;  
}

byte ReverseByte(byte in)
{
  byte out = 0;
  if (in & 0x01) out |= 0x80;
  if (in & 0x02) out |= 0x40;
  if (in & 0x04) out |= 0x20;
  if (in & 0x08) out |= 0x10;
  if (in & 0x10) out |= 0x08;
  if (in & 0x20) out |= 0x04;
  if (in & 0x40) out |= 0x02;
  if (in & 0x80) out |= 0x01;

  return out;
}


/*******************************************
/* SendSMS()
/* Hier werden die Wartezeiten über die Timerwerte realisiert
/* Damit das Programm nicht durch delay() blockert wird.
/*******************************************/
void SendSMS(byte bNbr, String sMsg)
{
 if(iSMS_SeqTimer == 25)
 {
  Serial3.print(F("AT\r\n"));
  //Serial.println(1);
 }
 if(iSMS_SeqTimer == 23) 
 {
  Serial3.print(F("AT+CFUN=1\r\n"));
  //Serial.println(2);
 }  
  
 if(iSMS_SeqTimer == 21)  
 {
  Serial3.print(F("AT+CPIN?\r\n"));
  //Serial.println(3);
 }  
 if(iSMS_SeqTimer == 19)
 {
  Serial3.print(F("AT+CMGF=1\r\n"));
  //Serial.println(4);
 } 
 if(iSMS_SeqTimer == 13)
 {
  Serial3.print(F("AT+CMGS="));
  Serial3.write(34);    // Anfühtungszeichen

  if(bNbr == 1)
   Serial3.print(SMS_Number1);  // 1. Nummer

  if(bNbr == 2)
   Serial3.print(SMS_Number2);  // 2. Nummer
      
  Serial3.write(34);    // Anführungszeichen
  Serial3.print("\r\n");
  //Serial.println(5);
   
 }
 
 if(iSMS_SeqTimer == 1)  // Null geht nicht, weil dann die Funktion SendSMS nicht aufgerufen wird
 {
  Serial3.print(sMsg);   // Nachricht
  Serial3.write(26);     // Zeichen "substitute" für Nachricht ende 
  //Serial.println(6);
   
 }
}
